datacoexp<-read.table("../Co-expression data.txt", header=T, sep="\t",stringsAsFactors=F)

tissues = read.table("../5 - tissue specific genes/tissueanalysis.txt", sep="\t", header=TRUE, stringsAsFactors=FALSE)
# Order the tissues according to the conservation of connectivity
ordtissres<- tissues[order(tissues$Conserved_coexpr_Diff_Loc, decreasing=F),]

#######Creation plot for conservation of connectivity##########
#Creation of vector with colors according to FDR values

#barplot creation
pdf("tissues_consconn.pdf",width = 6, height = 8)
consconn<-t(matrix(c(ordtissres$Conserved_coexpr_Diff_Loc_one, ordtissres$Conserved_coexpr_Diff_Loc), ncol=2))
contf <- t(matrix(c(ordtissres$FDR_Conserved_coexpr_one < 0.05, ordtissres$FDR_Conserved_coexpr < 0.05), ncol=2))
par(oma=c(0,8,0,0))
bp <- barplot(consconn, xlab="Difference in location", col=c("grey", "black"), beside=TRUE,
        horiz=T, space=c(0,1), names.arg=chartr(".", " ",ordtissres$name_tissue),  las=1)
text(-3, bp[consconn >0 & contf] , "*")
text(3, bp[consconn <0 & contf] , "*")
#to add a legend: legend =c("one-to-one only", "entire list"), args.legend = list(x = "bottomright")
#abline(v=0)
dev.off()


#######Creation plot for differential connectivity##########
#Creation of vector with colors according to FDR values

#barplot creation
pdf("tissues_diffconn.pdf",width = 3, height = 8)
diffconn<-t(matrix(c(ordtissres$Diff_Conn_Diff_Loc_one, ordtissres$Diff_Conn_Diff_Loc), ncol=2))
difftf <- t(matrix(c(ordtissres$FDR_Diff_Conn_one < 0.05, ordtissres$FDR_Diff_Conn < 0.05), ncol=2))
par(oma=c(0,0,0,0))
bp <- barplot(diffconn, xlab="Difference in location", col=c("grey", "black"), beside=TRUE,
        horiz=T, space=c(0,1))
text(-0.1, bp[diffconn >0 & difftf] , "*")
text(0.1, bp[diffconn <0 & difftf] , "*")
#to add a legend: legend =c("one-to-one only", "entire list"), args.legend = list(x = "bottomright")
#abline(v=0)
dev.off()



#######FOREST PLOTS FOR duplicated and homologous genes
library(forestplot)
pdf("tissues_dup.pdf",width = 2, height = 8)
forestplot(labeltext=rep(NA,nrow(ordtissres)), rev(ordtissres$ODDRatio_Dup),
           rev(ordtissres$dup_Conf_int_low), rev(ordtissres$dup_Conf_int_up), zero=1, clip=c(0.1,3.5),
           xticks = c(0.1,0.5, 1, 2, 3.5), xlog=T, new_page = F)
#forestplot(as.matrix(rev(ordtissres$name_tissue)), rev(ordtissres$ODDRatio_Dup), rev(ordtissres$dup_Conf_int_low), rev(ordtissres$dup_Conf_int_up), zero=1, clip=c(0,4))
dev.off()


pdf("tissues_hom.pdf",width = 2, height = 8)
forestplot(labeltext=rep(NA,nrow(ordtissres)), rev(ordtissres$ODDRatio_HOM_vs_nonHOM),
           rev(ordtissres$NonHom_Conf_int_low), rev(ordtissres$NonHom_Conf_int_up), zero=1, clip=c(0.1,2),
           xticks = c(0.1,0.5, 1, 2), xlog=T, new_page = F)
#forestplot(as.matrix(rev(ordtissres$name_tissue)), rev(ordtissres$ODDRatio_Dup), rev(ordtissres$dup_Conf_int_low), rev(ordtissres$dup_Conf_int_up), zero=1, clip=c(0,4))
dev.off()
