# load the libraries
libraries <- "tidyverse,RColorBrewer"
libraries <- strsplit(libraries, ",")[[1]]
for (name in libraries) {
  suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}

# load Reactine data
reactome <- read.delim("NCBI2Reactome_All_Levels.txt", comment.char = "", stringsAsFactors = F, sep = "\t",header = FALSE) # with readr
reactomeRel <- read_tsv("ReactomePathwaysRelation.txt", comment = "", col_names = F) # with readr

# filter out duplicated rows
reactALLHUM <- reactome %>% filter(V6 == "Homo sapiens") %>% select(V1,V2,V4) %>% distinct()  # to remove the rows that are duplicated where the same gene has been found manually and electronically

# tidy the data frame
reactALL <- reactALLHUM %>% select(V1,V2,V4) %>% nest(V1) %>% mutate(data = map(data, ~unlist(.x))) %>% mutate(len=map_int(data, ~length(.x)))   # here you have list columns
# reactALL <- unstack(reactHUM, X1~X2) # here you obtain a list where each element is a pathway, you could use X4 instead of X2
#

# if you want to find the patways related to immune system only
# # find all the immune system pathways
# reactALL %>% filter(V4 =="Immune System")
# childs <- reactomeRel %>% filter(X1 == "R-HSA-168256") %>% select(X2)
# temp <- c("R-HSA-168256", childs$X2)
# HierLevel <- 2
# termsIMMSYS <- cbind(temp, c(1, rep(HierLevel, length(childs$X2))))
# while(any(childs$X2 %in% reactomeRel$X1)){
#   HierLevel <- HierLevel + 1
#   childs <- reactomeRel %>% filter(X1 %in% childs$X2) %>% select(X2)
#   termsIMMSYS <- rbind( termsIMMSYS, cbind( childs$X2, rep(HierLevel, length(childs$X2)) ))
# }
# 


###### LIST DEFINITION FOR LATER ANALYSIS
reactLIST <- reactALL   # change it to reactIMM if you want only immune gene sets
######
# filter out terms with less than 5 genes
reactLIST <- reactLIST %>% filter(len > 3) 
write.table(reactLIST[,1:2], file="ReactomeList.txt", sep="\t", row.names=F, col.names=T)
save(reactLIST, file="reactLIST.RData")

# analysis pathways
datacoexp<-read.table("../Co-expression data.txt", header=T, sep="\t",stringsAsFactors=F)
entrez = read.table("../6 - pathway analysis/entrez IDs.txt", sep="\t", header=F, stringsAsFactors=FALSE)
########some parameter to set###########
colNAMES<-1   #value with gene names (for example the entrez ids)
homREL<-7   # column with homologous relationship types
colCOMM<-13 #column where there are the values of commonly co-expressed genes
colCOMMone<-16   #column commonly co-expressed genes One-to-one
colDIFF<-21   #column where there are the values of differential connectivity
TOTGENESHUM<-19727    #number of total genes from the original human co-expression map
HOMGENES<-length(unique(datacoexp[,colNAMES]))
nonHOMGENES<- TOTGENESHUM - HOMGENES
ONEpos<-which(datacoexp[,homREL] %in% "ortholog_one2one" == T)
datacoexpONE<-datacoexp[ONEpos,]



######## duplicated and non in human ##########
MANYd<-unique(datacoexp[datacoexp[,homREL] %in% "ortholog_many2many",colNAMES])    
MOUd<-unique(datacoexp[datacoexp[,homREL] %in% "ortholog_oneHUM2manyMOU",colNAMES])
HUMd<-unique(datacoexp[datacoexp[,homREL] %in% "ortholog_oneMOU2manyHUM",colNAMES])
ONE<-unique(datacoexp[datacoexp[,homREL] %in% "ortholog_one2one",colNAMES])   #genes not duplicated
DUP<-c(MANYd, HUMd, MOUd)   #genes duplicated

numONE<-length(ONE)  # length not duplicated 
numDUP<-length(DUP)  # length duplicated 


########Set up the variables to fill#############
name_pathway<-c()   # VECTOR 1
tot_genes<-c()   # VECTOR 2

number_Homologs<-c() # VECTOR 3
number_nonHomologs<-c()
ODDRatio_HOM_vs_nonHOM<-c()
P_Value_HOM_vs_nonHOM<-c()
NonHom_Conf_int_low<-c()
NonHom_Conf_int_up<-c()


one_to_one<-c()  # fisher test 
duplicated<-c()
ODDRatio_Dup<-c()
P_Value_Dup<-c()
dup_Conf_int_low<-c()
dup_Conf_int_up<-c()


Conserved_coexpr_Diff_Loc<-c()   # difference in location between the number of commonly co-expressed genes of the gene set and remaining genes 
Conserved_coexpr_P_Value<-c()     # pval for the mann-withey U test for the number of commonly co-expressed genes of the gene set and remaining genes
Conserved_coexpr_Diff_Loc_one<-c()  
Conserved_coexpr_P_Value_one<-c()     


Diff_Conn_Diff_Loc<-c()      # difference in location between the differential connectivity of the gene set and remaining genes 
Diff_Conn_P_Value<-c()     # pval for the mann-withey U test for the differential connectivity of the gene set and remaining genes
Diff_Conn_Diff_Loc_one<-c() 
Diff_Conn_P_Value_one<-c()


###### Loop over a file connection##### the first part is specific to the file opened
for(i in 1:nrow(reactLIST)){
  vect<-reactLIST[i,]$data[[1]]   
  
  name_pathway<-c(name_pathway, reactLIST[i,]$V4)   # VECTOR 1 it add the name of the pathway each time  
  
  SET<-as.numeric(vect)     # it gives the vector with all genes contained in a pathway
  SET<-SET[!is.na(SET)]
  SET<-intersect(entrez[,1], SET)
  ###from here should be equal among tissue, pathways and diseases####
  tot_genes<-c(tot_genes, length(SET))   # VECTOR 2 
  homSET<-unique(datacoexp[datacoexp[,colNAMES]%in%SET,colNAMES])   # genes in the pathway that are orthologs
  number_Homologs<-c(number_Homologs, length(homSET))   # VECTOR 3
  number_nonHomologs<-c(number_nonHomologs, length(SET)-length(homSET))
  #fisher test Homologous versus non Homologous
  matrixHOMnonHOM<-matrix(data = c(HOMGENES-length(homSET), nonHOMGENES-(length(SET)-length(homSET)), length(homSET), length(SET)-length(homSET)), nrow = 2, ncol = 2, byrow = FALSE)
  testfHOMnonHOM<-fisher.test(matrixHOMnonHOM)
  ODDRatio_HOM_vs_nonHOM<-c(ODDRatio_HOM_vs_nonHOM, as.numeric(testfHOMnonHOM[3]))
  P_Value_HOM_vs_nonHOM<-c(P_Value_HOM_vs_nonHOM, as.numeric(testfHOMnonHOM[1]))
  NonHom_Conf_int_low<-c(NonHom_Conf_int_low, testfHOMnonHOM[[2]][1])
  NonHom_Conf_int_up<-c(NonHom_Conf_int_up, testfHOMnonHOM[[2]][2])
  
  
  #fisher test for duplicated genes
  ONEset<-ONE[ONE %in% SET]     # not duplicated
  DUPset<-DUP[DUP %in% SET]     # duplicated in human
  contHUM<-matrix(data = c(numONE-length(ONEset), numDUP-length(DUPset), length(ONEset), length(DUPset)), nrow = 2, ncol = 2, byrow = FALSE)  # contingency table 
  testfHUM<-fisher.test(contHUM)
  
  #fill the variables
  one_to_one<-c(one_to_one, length(ONEset))
  duplicated<-c(duplicated, length(DUPset))
  ODDRatio_Dup<-c(ODDRatio_Dup, as.numeric(testfHUM[3]))
  P_Value_Dup<-c(P_Value_Dup, as.numeric(testfHUM[1]))
  dup_Conf_int_low<-c(dup_Conf_int_low, testfHUM[[2]][1])
  dup_Conf_int_up<-c(dup_Conf_int_up, testfHUM[[2]][2])
  
  
  # testing connectivity in terms of number of commonly coexpressed genes
  commcoexpvaluesSET<-datacoexp[datacoexp[,colNAMES]%in%SET,colCOMM]     # number of commonly co-expressed genes of the gene set
  commcoexpvaluesREM<-datacoexp[!datacoexp[,colNAMES]%in%SET,colCOMM]       # number of commonly co-expressed genes of the remaining genes
  testCOMMON<-wilcox.test(commcoexpvaluesSET, commcoexpvaluesREM, conf.int=T)
  Conserved_coexpr_Diff_Loc<-c(Conserved_coexpr_Diff_Loc, testCOMMON[[9]])  
  Conserved_coexpr_P_Value<-c(Conserved_coexpr_P_Value, as.numeric(testCOMMON[[3]])) 
  
  # testing connectivity in terms of number of commonly coexpressed genes considering only homologs with one-to-one relationship
  commcoexpvaluesSET<-datacoexpONE[datacoexpONE[,colNAMES]%in%SET,colCOMMone]     # number of commonly co-expressed genes of the gene set
  commcoexpvaluesREM<-datacoexpONE[!datacoexpONE[,colNAMES]%in%SET,colCOMMone]       # number of commonly co-expressed genes of the remaining genes
  testCOMMON<-wilcox.test(commcoexpvaluesSET, commcoexpvaluesREM, conf.int=T)
  Conserved_coexpr_Diff_Loc_one<-c(Conserved_coexpr_Diff_Loc_one, testCOMMON[[9]])  
  Conserved_coexpr_P_Value_one<-c(Conserved_coexpr_P_Value_one, as.numeric(testCOMMON[[3]])) 
  
  # testing differnce in differential connectivity
  DiffConnvaluesSET<-datacoexp[datacoexp[,colNAMES]%in%SET,colDIFF]     # values of differential connectivity of the gene set
  DiffConnvaluesREM<-datacoexp[!datacoexp[,colNAMES]%in%SET,colDIFF]       # values of differential connectivity of the remaining genes
  testDIFF<-wilcox.test(DiffConnvaluesSET, DiffConnvaluesREM, conf.int=T)
  Diff_Conn_Diff_Loc<-c(Diff_Conn_Diff_Loc, testDIFF[[9]])  
  Diff_Conn_P_Value<-c(Diff_Conn_P_Value, as.numeric(testDIFF[[3]])) 
  
  
  # testing differnce in differential connectivity considering only one_to_one
  DiffConnvaluesSET<-datacoexpONE[datacoexpONE[,colNAMES]%in%SET,colDIFF]     # values of differential connectivity of the gene set
  DiffConnvaluesREM<-datacoexpONE[!datacoexpONE[,colNAMES]%in%SET,colDIFF]       # values of differential connectivity of the remaining genes
  testDIFF<-wilcox.test(DiffConnvaluesSET, DiffConnvaluesREM, conf.int=T)
  Diff_Conn_Diff_Loc_one<-c(Diff_Conn_Diff_Loc_one, testDIFF[[9]])  
  Diff_Conn_P_Value_one<-c(Diff_Conn_P_Value_one, as.numeric(testDIFF[[3]])) 
  
  print(i)
}


FDR_HOM_vs_nonHOM<-p.adjust(P_Value_HOM_vs_nonHOM, "fdr")
FDR_Dup<-p.adjust(P_Value_Dup, "fdr")
FDR_Conserved_coexpr<-p.adjust(Conserved_coexpr_P_Value, "fdr")
FDR_Conserved_coexpr_one<-p.adjust(Conserved_coexpr_P_Value_one, "fdr")
FDR_Diff_Conn<-p.adjust(Diff_Conn_P_Value, "fdr")
FDR_Diff_Conn_one<-p.adjust(Diff_Conn_P_Value_one, "fdr")


#values I am interested in:
res<- data.frame(name_pathway, tot_genes, number_Homologs, number_nonHomologs, ODDRatio_HOM_vs_nonHOM, 
                 P_Value_HOM_vs_nonHOM, FDR_HOM_vs_nonHOM, NonHom_Conf_int_low, NonHom_Conf_int_up, 
                 one_to_one, duplicated, ODDRatio_Dup, P_Value_Dup, FDR_Dup, dup_Conf_int_low, dup_Conf_int_up,
                 Conserved_coexpr_Diff_Loc, Conserved_coexpr_P_Value,  FDR_Conserved_coexpr, 
                 Conserved_coexpr_Diff_Loc_one, Conserved_coexpr_P_Value_one,  FDR_Conserved_coexpr_one,                  
                 Diff_Conn_Diff_Loc, Diff_Conn_P_Value, FDR_Diff_Conn,
                 Diff_Conn_Diff_Loc_one, Diff_Conn_P_Value_one, FDR_Diff_Conn_one
)

rownames(res) <- reactLIST$V2
# order the list according to reactome hierarchy
# NOdupTEMP <- termsIMMSYS[!duplicated(termsIMMSYS[,1]),]
# res <- res[NOdupTEMP[,1],]


write.table(res, file="ReactomeAnalysis.txt", sep="\t", row.names=F, col.names=T)

reactAnal<-res
save(reactAnal, file="ReactomeAnal.RData")




####### READING database with Base functions ######
# If you want to use base functions remember to use the argument quote="" otherwise you get less rows # check https://www.biostars.org/p/221983/ for the explanation
# readr is faster though
# reactALL <- read.delim("Ensembl2Reactome_All_Levels.txt",blank.lines.skip = FALSE,check.names = FALSE,stringsAsFactors = F, sep = "\t",header = FALSE, comment.char = "")
# reactALL <- read.table("Ensembl2Reactome_All_Levels.txt",stringsAsFactors = F, sep = "\t", comment.char = "", quote="")


