# load the libraries
libraries <- "tidyverse,RColorBrewer,forestplot"
libraries <- strsplit(libraries, ",")[[1]]
for (name in libraries) {
  suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}

load("ReactomeAnal.RData")
reactomeRel <- read_tsv("ReactomePathwaysRelation.txt", comment = "", col_names = F) # with readr


### SELECT IMMUNE SYSTEM PATHWAYS
childs <- reactomeRel %>% filter(X1 == "R-HSA-168256") %>% select(X2)
path <- "R-HSA-168256"
# for(i in childs$X2){  # only the second hiearchy level
#   path <- c(path, i, reactomeRel %>% filter(X1 %in% i) %>% select(X2) %>% unlist())
# }
for(i in childs$X2){   # for third levels
  path <- c(path, i)
  childs2 <- reactomeRel %>% filter(X1 %in% i) %>% select(X2) %>% unlist()
  for(a in childs2){
    path <- c(path, a, reactomeRel %>% filter(X1 %in% a) %>% select(X2) %>% unlist())
  }
}


ReactSEL <- reactAnal[path,]
ReactSEL <- ReactSEL[!is.na(ReactSEL[,1]), ]

# barplot commonly co-expressed genes
consconn<-t(matrix(c(rev(ReactSEL$Conserved_coexpr_Diff_Loc_one), rev(ReactSEL$Conserved_coexpr_Diff_Loc)), ncol=2))
contf <- t(matrix(c(rev(ReactSEL$FDR_Conserved_coexpr_one) < 0.05, rev(ReactSEL$FDR_Conserved_coexpr) < 0.05), ncol=2))
pdf(file = "IMMconservedCoexp.pdf",width = 11, height = 20)
par(oma=c(0,35,0,0))
bp<- barplot(consconn, xlab="Difference in location", col=c("grey", "black"), beside=TRUE,
        horiz=T, space=c(0,2), names.arg=rev(chartr(".", " ", ReactSEL$name_pathway)),  las=1)
text(-4, bp[consconn >0 & contf] , "*")
text(4, bp[consconn <0 & contf] , "*")
dev.off()

# barplot differential connectivity
diffconn<-t(matrix(c(rev(ReactSEL$Diff_Conn_Diff_Loc_one), rev(ReactSEL$Diff_Conn_Diff_Loc)), ncol=2))
difftf <- t(matrix(c(rev(ReactSEL$FDR_Diff_Conn_one) < 0.05, rev(ReactSEL$FDR_Diff_Conn) < 0.05), ncol=2))
pdf(file = "IMMdifferentialConnectivity.pdf",width = 3, height = 20)
par(oma=c(0,0,0,0))
bp<- barplot(diffconn, xlab="Difference in location", col=c("grey", "black"), beside=TRUE,
             horiz=T, space=c(0,2),  las=1,legend =c("one-to-one only", "entire list"), args.legend = list(x = "bottomright"))
text(-0.5, bp[diffconn >0 & difftf] , "*")
text(0.5, bp[diffconn <0 & difftf] , "*")
dev.off()


# duplicated genes
pdf(file = "IMMduplicated.pdf",width = 2.5, height = 20)
forestplot(labeltext=rep(NA,nrow(ReactSEL)), ReactSEL$ODDRatio_Dup,
           ReactSEL$dup_Conf_int_low, ReactSEL$dup_Conf_int_up, zero=1, clip=c(0.1,30), xlog=T,new_page=F)
dev.off()


# non homologous genes
pdf(file = "IMMnonHomolog.pdf",width = 2.5, height = 20)
forestplot(labeltext=rep(NA,nrow(ReactSEL)), ReactSEL$ODDRatio_HOM_vs_nonHOM,
           ReactSEL$NonHom_Conf_int_low, ReactSEL$NonHom_Conf_int_up, zero=1, clip=c(0.1,30), xlog=T,new_page=F)
dev.off()

