# load the libraries
options(stringsAsFactors = FALSE)
libraries <- "tidyverse,RColorBrewer,forestplot"
libraries <- strsplit(libraries, ",")[[1]]
for (name in libraries) {
  suppressPackageStartupMessages(library(name, character.only=T, warn.conflicts=F, quietly=T, verbose=F))
}

load("ReactomeAnal.RData")
reactomeRel <- read_tsv("ReactomePathwaysRelation.txt", comment = "", col_names = F) # with readr

reactomeRel <- reactomeRel[grep("HSA", reactomeRel$X1),]
### select all parent patheways
rootPath <- unique(reactomeRel[which(!reactomeRel$X1 %in% reactomeRel$X2),]$X1)
# order them by alphabetic order
rootPath <- rootPath[order(as.character(reactAnal[rootPath,]$name_pathway))]

# find hierarchy
ReactOrd <- c()
for(l1 in rootPath){
  ReactOrd <- rbind( ReactOrd, c( l1,l1,l1, 1, reactAnal[l1,]$name_pathway))  
  level2 <- reactomeRel %>% filter(X1 %in% l1) %>% select(X2) %>% unlist()
  for(l2 in level2){
    ReactOrd <- rbind( ReactOrd, c( l1,l2,l2, 2, reactAnal[l2,]$name_pathway))
    childs <- reactomeRel %>% filter(X1 %in% l2) %>% select(X2) %>% unlist()
    temp <-childs
    if( length(temp) != 0 ) {
      while(any(childs %in% reactomeRel$X1)){
        childs <- reactomeRel %>% filter(X1 %in% childs) %>% select(X2) %>% unlist()
        temp <- c( temp, childs )
      }
      temp <- unique(temp)
      len <-length(temp)
      ReactOrd <- rbind( ReactOrd, cbind( rep(l1, len ),rep(l2, len),temp, rep(3, len), reactAnal[temp,]$name_pathway))
    }
  }
}
ReactOrd <- ReactOrd[!is.na(ReactOrd[,5]),]
colnames(ReactOrd) <- c("l1id", "l2id", "l3id","level", "name")

lev3 <- ReactOrd[which(ReactOrd[,4] == 3),3]
torem <- unique(lev3[lev3 %in% ReactOrd[which(ReactOrd[,4] == 2),3]])
ReactOrd2 <- ReactOrd[-which(ReactOrd[,4] == 3 & ReactOrd[,3] %in% torem),]
ReactOrd2 <- ReactOrd2[!duplicated(ReactOrd2[,3]),]
save(file = "React_ordered.RData", ReactOrd2)

# remove all the pathways of the immune system
ReactOrd2 <- ReactOrd2[!ReactOrd2[,1] %in% c("R-HSA-168256"), ]   # remove immune system and diseases

# select third or higher levels pathways
namesl3 <- ReactOrd2[which(ReactOrd2[,4] == 3), 3] # names level 3
FDRdata <- reactAnal[namesl3, c(7,14,19,22,25,28)]  # take columns with FDR values
# thres <- 0.05
# idx2 <- apply(FDRdata, 1, function(x) sum(c(x[1] < thres , x[2]<thres , any(x[c(3,4)] <thres ),  any(x[c(5,6)] <thres )) )>=2  | sum(x < 0.00000000005 )>=1) # this way was maybe better
 idx2 <- apply(FDRdata, 1, function(x) sum(x < 0.05 )>=4 |  sum(x < 0.00000000005 )>=1)
sum(idx2)
reactAnal[rownames(FDRdata[idx2,]),]$name_pathway

path <- ReactOrd2[which(ReactOrd2[,3] %in% c(ReactOrd2[which(ReactOrd2[,4] %in% c(1,2)), 3], rownames(FDRdata[idx2,]))), 3]

ReactSEL <- reactAnal[path,]
height1 <- 20/99*nrow(ReactSEL)
height1
# barplot commonly co-expressed genes
consconn<-t(matrix(c(rev(ReactSEL$Conserved_coexpr_Diff_Loc_one), rev(ReactSEL$Conserved_coexpr_Diff_Loc)), ncol=2))
contf <- t(matrix(c(rev(ReactSEL$FDR_Conserved_coexpr_one) < 0.05, rev(ReactSEL$FDR_Conserved_coexpr) < 0.05), ncol=2))
pdf(file = "ALLconservedCoexp.pdf",width = 11, height = height1)
par(oma=c(0,35,0,0))
bp<- barplot(consconn, xlab="Difference in location", col=c("grey", "black"), beside=TRUE,
             horiz=T, space=c(0,2), names.arg=rev(chartr(".", " ", ReactSEL$name_pathway)),  las=1)
text(-5.5, bp[consconn >0 & contf] , "*")
text(5.5, bp[consconn <0 & contf] , "*")
dev.off()

# barplot differential connectivity
diffconn<-t(matrix(c(rev(ReactSEL$Diff_Conn_Diff_Loc_one), rev(ReactSEL$Diff_Conn_Diff_Loc)), ncol=2))
difftf <- t(matrix(c(rev(ReactSEL$FDR_Diff_Conn_one) < 0.05, rev(ReactSEL$FDR_Diff_Conn) < 0.05), ncol=2))
pdf(file = "ALLdifferentialConnectivity.pdf",width = 3, height = height1)
par(oma=c(0,0,0,0))
bp<- barplot(diffconn, xlab="Difference in location", col=c("grey", "black"), beside=TRUE,
             horiz=T, space=c(0,2),  las=1,legend =c("one-to-one only", "entire list"), args.legend = list(x = "bottomright"))
text(-0.4, bp[diffconn >0 & difftf] , "*")
text(0.4, bp[diffconn <0 & difftf] , "*")
dev.off()


# duplicated genes
pdf(file = "ALLduplicated.pdf",width = 2.5, height = height1)
forestplot(labeltext=rep(NA,nrow(ReactSEL)), ReactSEL$ODDRatio_Dup,
           ReactSEL$dup_Conf_int_low, ReactSEL$dup_Conf_int_up, zero=1, clip=c(0.1,30), xlog=T,new_page=F)
dev.off()


# non homologous genes
pdf(file = "ALLnonHomolog.pdf",width = 2.5, height = height1)
forestplot(labeltext=rep(NA,nrow(ReactSEL)), ReactSEL$ODDRatio_HOM_vs_nonHOM,
           ReactSEL$NonHom_Conf_int_low, ReactSEL$NonHom_Conf_int_up, zero=1, clip=c(0.1,30), xlog=T,new_page=F)
dev.off()


# first way for selecting pathways
# ### Remove unwanted pathways
# rootPath <- rootPath[!rootPath %in% c("R-HSA-168256")]   # remove immune system and diseases
# 
# 
# ###find second level pathways
# path <- c()
# for(i in rootPath){  # only the second hiearchy level
#   path <- c(path, i, reactomeRel %>% filter(X1 %in% i) %>% select(X2) %>% unlist())
# }
# 
# # add OLFACTORY SYSTEM
# path2 <- c(path[1:which(path == "R-HSA-372790")], "R-HSA-381753", path[(which(path == "R-HSA-372790")+1):length(path)])
# # subset data
# ReactSEL <- reactAnal[path2,]
# ReactSEL <- ReactSEL[!is.na(ReactSEL[,1]), ]
# #filter out small gene sets
# #ReactSEL <- ReactSEL[ReactSEL$tot_genes > 5, ]



# I started modifying the Signaling transduction pathways but then I thought it was not necessary. In case you need the code see here:
# childs <- reactomeRel %>% filter(X1 == "R-HSA-162582") %>% select(X2)
# termSIG <- childs$X2
# while(any(childs$X2 %in% reactomeRel$X1)){
#   childs <- reactomeRel %>% filter(X1 %in% childs$X2) %>% select(X2)
#   termSIG <- c( termsSIG, childs$X2 )
# }
# FDRtermSIG <- reactAnal[unique(termSIG),c(7,14,19,22,25,28)]
# FDRtermSIG <- FDRtermSIG[!is.na(FDRtermSIG[,1]), ]
# minFDR <- sort(apply(FDRtermSIG, 1, min))


