# load data with gene homology information
datacoexp<-read.table("Co-expression data.txt", header=T, sep="\t",stringsAsFactors=F)

#load gene information
symbols = read.table("gene symbols.txt", sep="\t", header=F, stringsAsFactors=FALSE)
## File
file <- "NIA_human_GAD-GeneSets_2010-21-08_modified.txt"
## Create connection
con <- file(description=file, open="r")

## Hopefully you know the number of lines from some other source or
num<-count.fields(file,sep="\t")


########some parameter to set###########
colNAMES<-5   #value with gene names (for example the entrez ids)
homREL<-7   # column with homologous relationship values
colCOMM<-13 #column where there are the values of commonly co-expressed genes
colCOMMone<-16   #column commonly co-expressed genes One-to-one
colDIFF<-19   #column where there are the values of differential connectivity
TOTGENESHUM<-19727    #number of total genes from the original human co-expression map
HOMGENES<-length(unique(datacoexp[,colNAMES]))
nonHOMGENES<- TOTGENESHUM - HOMGENES
ONEpos<-which(datacoexp[,homREL] %in% "ortholog_one2one" == T)
datacoexpONE<-datacoexp[ONEpos,]

######## duplicated and non in human ##########
MANYd<-unique(datacoexp[datacoexp[,homREL] %in% "ortholog_many2many",colNAMES])    
MOUd<-unique(datacoexp[datacoexp[,homREL] %in% "ortholog_oneHUM2manyMOU",colNAMES])
HUMd<-unique(datacoexp[datacoexp[,homREL] %in% "ortholog_oneMOU2manyHUM",colNAMES])
ONE<-unique(datacoexp[datacoexp[,homREL] %in% "ortholog_one2one",colNAMES])   #genes not duplicated
DUP<-c(MANYd, HUMd, MOUd)   #genes duplicated

numONE<-length(ONE)  # length not duplicated 
numDUP<-length(DUP)  # length duplicated 


########Set up the variables to fill#############
name_disease<-c()   # VECTOR 1
meshterm<-c()
tot_genes<-c()   # VECTOR 2

number_Homologs<-c() # VECTOR 3
number_nonHomologs<-c()
ODDRatio_HOM_vs_nonHOM<-c()
P_Value_HOM_vs_nonHOM<-c()
NonHom_Conf_int_low<-c()
NonHom_Conf_int_up<-c()


one_to_one<-c()  # fisher test 
duplicated<-c()
ODDRatio_Dup<-c()
P_Value_Dup<-c()
dup_Conf_int_low<-c()
dup_Conf_int_up<-c()


Conserved_coexpr_Diff_Loc<-c()   # difference in location between the number of commonly co-expressed genes of the gene set and remaining genes 
Conserved_coexpr_P_Value<-c()     # pval for the mann-withey U test for the number of commonly co-expressed genes of the gene set and remaining genes
Conserved_coexpr_Diff_Loc_one<-c()  
Conserved_coexpr_P_Value_one<-c()     


Diff_Conn_Diff_Loc<-c()      # difference in location between the differential connectivity of the gene set and remaining genes 
Diff_Conn_P_Value<-c()     # pval for the mann-withey U test for the differential connectivity of the gene set and remaining genes
Diff_Conn_Diff_Loc_one<-c() 
Diff_Conn_P_Value_one<-c()

###### Loop over a file connection##### the first part is specific to the file opened
for(i in 1:length(num)) {
  tmp <- scan(file=con, nlines=1, quiet=TRUE, sep="\t", what=list("character"))
  SET<-strsplit(tmp[[1]][4], ",")[[1]]
  SET<-intersect(symbols[,1], SET)
  if(length(SET) >= 10){
    name_disease<-c(name_disease, tmp[[1]][2])   # VECTOR 1 it add the name of the disease each time  
    meshterm<-c(meshterm, tmp[[1]][3])
    
    ###from here should be equal among tissue, pathways and diseases####
    tot_genes<-c(tot_genes, length(SET))   # VECTOR 2 
    homSET<-unique(datacoexp[datacoexp[,colNAMES]%in%SET,colNAMES])   # genes in the pathway that are orthologs
    number_Homologs<-c(number_Homologs, length(homSET))   # VECTOR 3
    number_nonHomologs<-c(number_nonHomologs, length(SET)-length(homSET))
    #fisher test Homologous versus non Homologous
    matrixHOMnonHOM<-matrix(data = c(HOMGENES-length(homSET), nonHOMGENES-(length(SET)-length(homSET)), length(homSET), length(SET)-length(homSET)), nrow = 2, ncol = 2, byrow = FALSE)
    testfHOMnonHOM<-fisher.test(matrixHOMnonHOM)
    ODDRatio_HOM_vs_nonHOM<-c(ODDRatio_HOM_vs_nonHOM, as.numeric(testfHOMnonHOM[3]))
    P_Value_HOM_vs_nonHOM<-c(P_Value_HOM_vs_nonHOM, as.numeric(testfHOMnonHOM[1]))
    NonHom_Conf_int_low<-c(NonHom_Conf_int_low, testfHOMnonHOM[[2]][1])
    NonHom_Conf_int_up<-c(NonHom_Conf_int_up, testfHOMnonHOM[[2]][2])
    
    
    #fisher test for duplicated genes
    ONEset<-ONE[ONE %in% SET]     # not duplicated
    DUPset<-DUP[DUP %in% SET]     # duplicated in human
    contHUM<-matrix(data = c(numONE-length(ONEset), numDUP-length(DUPset), length(ONEset), length(DUPset)), nrow = 2, ncol = 2, byrow = FALSE)  # contingency table 
    testfHUM<-fisher.test(contHUM)
    
    #fill the variables
    one_to_one<-c(one_to_one, length(ONEset))
    duplicated<-c(duplicated, length(DUPset))
    ODDRatio_Dup<-c(ODDRatio_Dup, as.numeric(testfHUM[3]))
    P_Value_Dup<-c(P_Value_Dup, as.numeric(testfHUM[1]))
    dup_Conf_int_low<-c(dup_Conf_int_low, testfHUM[[2]][1])
    dup_Conf_int_up<-c(dup_Conf_int_up, testfHUM[[2]][2])
    
    
    # testing connectivity in terms of number of commonly coexpressed genes
    commcoexpvaluesSET<-datacoexp[datacoexp[,colNAMES]%in%SET,colCOMM]     # number of commonly co-expressed genes of the gene set
    commcoexpvaluesREM<-datacoexp[!datacoexp[,colNAMES]%in%SET,colCOMM]       # number of commonly co-expressed genes of the remaining genes
    testCOMMON<-wilcox.test(commcoexpvaluesSET, commcoexpvaluesREM, conf.int=T)
    Conserved_coexpr_Diff_Loc<-c(Conserved_coexpr_Diff_Loc, testCOMMON[[9]])  
    Conserved_coexpr_P_Value<-c(Conserved_coexpr_P_Value, as.numeric(testCOMMON[[3]])) 
    
    # testing connectivity in terms of number of commonly coexpressed genes considering only homologs with one-to-one relationship
    commcoexpvaluesSET<-datacoexpONE[datacoexpONE[,colNAMES]%in%SET,colCOMMone]     # number of commonly co-expressed genes of the gene set
    commcoexpvaluesREM<-datacoexpONE[!datacoexpONE[,colNAMES]%in%SET,colCOMMone]       # number of commonly co-expressed genes of the remaining genes
    testCOMMON<-wilcox.test(commcoexpvaluesSET, commcoexpvaluesREM, conf.int=T)
    Conserved_coexpr_Diff_Loc_one<-c(Conserved_coexpr_Diff_Loc_one, testCOMMON[[9]])  
    Conserved_coexpr_P_Value_one<-c(Conserved_coexpr_P_Value_one, as.numeric(testCOMMON[[3]])) 
    
    # testing differnce in differential connectivity
    DiffConnvaluesSET<-datacoexp[datacoexp[,colNAMES]%in%SET,colDIFF]     # values of differential connectivity of the gene set
    DiffConnvaluesREM<-datacoexp[!datacoexp[,colNAMES]%in%SET,colDIFF]       # values of differential connectivity of the remaining genes
    testDIFF<-wilcox.test(DiffConnvaluesSET, DiffConnvaluesREM, conf.int=T)
    Diff_Conn_Diff_Loc<-c(Diff_Conn_Diff_Loc, testDIFF[[9]])  
    Diff_Conn_P_Value<-c(Diff_Conn_P_Value, as.numeric(testDIFF[[3]])) 
    
    
    # testing differnce in differential connectivity considering only one_to_one
    DiffConnvaluesSET<-datacoexpONE[datacoexpONE[,colNAMES]%in%SET,colDIFF]     # values of differential connectivity of the gene set
    DiffConnvaluesREM<-datacoexpONE[!datacoexpONE[,colNAMES]%in%SET,colDIFF]       # values of differential connectivity of the remaining genes
    testDIFF<-wilcox.test(DiffConnvaluesSET, DiffConnvaluesREM, conf.int=T)
    Diff_Conn_Diff_Loc_one<-c(Diff_Conn_Diff_Loc_one, testDIFF[[9]])  
    Diff_Conn_P_Value_one<-c(Diff_Conn_P_Value_one, as.numeric(testDIFF[[3]])) 
  }
}

close(con)
  
  FDR_HOM_vs_nonHOM<-p.adjust(P_Value_HOM_vs_nonHOM, "fdr")
  FDR_Dup<-p.adjust(P_Value_Dup, "fdr")
  FDR_Conserved_coexpr<-p.adjust(Conserved_coexpr_P_Value, "fdr")
  FDR_Conserved_coexpr_one<-p.adjust(Conserved_coexpr_P_Value_one, "fdr")
  FDR_Diff_Conn<-p.adjust(Diff_Conn_P_Value, "fdr")
  FDR_Diff_Conn_one<-p.adjust(Diff_Conn_P_Value_one, "fdr")
  
  
  #values I am interested in:
  res<- data.frame(name_disease, meshterm, tot_genes, number_Homologs, number_nonHomologs, ODDRatio_HOM_vs_nonHOM, 
                   P_Value_HOM_vs_nonHOM, FDR_HOM_vs_nonHOM, NonHom_Conf_int_low, NonHom_Conf_int_up, 
                   one_to_one, duplicated, ODDRatio_Dup, P_Value_Dup, FDR_Dup, dup_Conf_int_low, dup_Conf_int_up,
                   Conserved_coexpr_Diff_Loc, Conserved_coexpr_P_Value,  FDR_Conserved_coexpr, 
                   Conserved_coexpr_Diff_Loc_one, Conserved_coexpr_P_Value_one,  FDR_Conserved_coexpr_one,                  
                   Diff_Conn_Diff_Loc, Diff_Conn_P_Value, FDR_Diff_Conn,
                   Diff_Conn_Diff_Loc_one, Diff_Conn_P_Value_one, FDR_Diff_Conn_one
  )

write.table(res, file="diseasesanalysis.txt", sep="\t", row.names=F, col.names=T)

diseasesanalysis<-res
save(diseasesanalysis, file="diseasesanalysis")