# load data with gene homology information
datacoexp<-read.table("Co-expression data.txt", header=T, sep="\t",stringsAsFactors=F)

# load data with gene information
entrezID<-read.table("entrezID.txt", header=T, sep="\t")
# load networks
degreeHUM<-read.table("degreeHUM1p.txt", header=F, sep="\t")
degreeMOU<-read.table("degreeMOU1p.txt", header=F, sep="\t")

#### calculation of differential connected genes #############
results<-c()
for(i in 1:length(entrezID[,1])){
  kHUMgene<-degreeHUM[degreeHUM[,1]%in%entrezID[i,1],2]
  kMOUgene<-degreeMOU[degreeMOU[,1]%in%entrezID[i,2],2]
  if(length(kHUMgene)!=0 && length(kMOUgene)!=0){
    normKhum<-kHUMgene/length(degreeHUM[,1])
    normKmou<-kMOUgene/length(degreeMOU[,1])
    diffK<-normKhum-normKmou
  }
  else{
    diffK<-"NA"
  }
  results<-rbind(results, c(entrezID[i,1], entrezID[i,2], diffK))
}


write.table(results, file="diffconnectivityhom1p.txt", sep="\t", row.names=F, col.names=F)

#### comparison of DC genes and commonly coexpressed genes and dn/ds ratio ######
datacoexp[which(datacoexp[,17]>= 21.99), 10]
datacoexp[which(datacoexp[,17]<= 0.039), 10]
boxplot(datacoexp[,13], datacoexp[which(datacoexp[,17]>= 21.99), 13], datacoexp[which(datacoexp[,17]<= 0.039), 13], 
        main="Genes commonly co-expressed in differentially Connected genes", ylab="Commonly co-expressed genes",
        names=c("all genes", "Highly DC in Human","Highly DC in Mouse"))
boxplot(datacoexp[,10], datacoexp[which(datacoexp[,17]>= 21.99), 10], datacoexp[which(datacoexp[,17]<= 0.039), 10], 
        main="Dn/ds ratio in differentially Connected genes", ylab="dn/ds ratio", 
        names=c("all genes", "Highly DC in Human","Highly DC in Mouse"))


##### to perform the fisher test on orthology relationship #############
table(datacoexp[,7])
table(datacoexp[which(datacoexp[,19]>= 0.05),7])
table(datacoexp[which(datacoexp[,19]<= -0.05),7])
table(datacoexp[which(datacoexp[,19]> -0.05 & datacoexp[,19]< 0.05),7])
rownam<-list(c("one-to-many and many-to-many orthologs", "one-to-one orthologs"))
#manually inserts the values from the tables:
cont<-matrix(data = c(78,285, 1998,14086), nrow = 2, ncol = 2, byrow = FALSE)
chisq.test(cont)
fisher.test(cont)


###############   fisher test 2nd method   #################
numONE<-length(datacoexp[datacoexp[,7]%in%"ortholog_one2one",1])
numONEtoMANY<-length(unique(datacoexp[datacoexp[,7]%in%"ortholog_one2many",1]))
numMANYtoMANY<-length(unique(datacoexp[datacoexp[,7]%in%"ortholog_many2many",1]))
numMANY<-numONEtoMANY + numMANYtoMANY
## to continue

